/*
 *  Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 *  Licensed under the Eclipse Public License - v 1.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *           http://www.eclipse.org/legal/epl-v10.html
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package nl.cloudfarming.client.field.type;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTextField;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.field.model.SoilType;
import org.jdesktop.beansbinding.*;
import org.jdesktop.swingx.JXDatePicker;
import org.jdesktop.swingx.JXLabel;
import org.jdesktop.swingx.JXTextField;
import org.jdesktop.swingx.combobox.EnumComboBoxModel;
import org.jdesktop.swingx.combobox.ListComboBoxModel;
import org.openide.util.NbBundle;

/**
 * Panel which displays the data of a field
 *
 * @author Merijn Zengers
 */
@NbBundle.Messages("unit abbreviation hectares=ha")
public class FieldDataPanel extends javax.swing.JPanel {

    private BeanProperty nameTextField = BeanProperty.create("text");
    private BeanProperty fieldPropName = BeanProperty.create(Field.PROP_NAME);
    private Binding nameBinding;
    private BeanProperty textProperty = BeanProperty.create("text");
    private BeanProperty fieldPropArea = BeanProperty.create(Field.PROP_AREA_IN_HECTARES);
    private Binding areaBinding;
    private BeanProperty formPropValidityStart = BeanProperty.create("date");
    private BeanProperty fieldPropStartDate = BeanProperty.create(Field.PROP_START_DATE);
    private Binding validityStartBinding;
    private BeanProperty formPropValidityEnd = BeanProperty.create("date");
    private BeanProperty fieldPropEndDate = BeanProperty.create(Field.PROP_END_DATE);
    private Binding validityEndBinding;
    private BeanProperty formPropSoilType = BeanProperty.create("selectedItem");
    private BeanProperty fieldPropSoilType = BeanProperty.create(Field.PROP_SOIL_TYPE);
    private Binding soilTypeBinding;
    private BindingGroup fieldBindingGroup = new BindingGroup();
    private final Field field;
    private ListComboBoxModel<SoilType> soilTypeModel = new EnumComboBoxModel<>(SoilType.class);

    /**
     * Creates new form FieldDataPanel2
     */
    public FieldDataPanel(Field field) {
        assert field != null : "Cannot construct a FieldDataPanel with a null field";
        this.field = field;
        initComponents();
        initValues();
        bind();
    }

    /**
     * Initiate field values to values in the Field object
     */
    private void initValues() {
        nameField.setText(field.getName());
        areaField.setText(String.valueOf(field.getAreaInHectares()));
        validityStartField.setDate(field.getStartDate());
        validityEndField.setDate(field.getEndDate());
        soilTypeModel.setSelectedItem(field.getSoilType());
    }

    /**
     * bind form fields to Field properties
     */
    private void bind() {
        nameBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE,
                field, fieldPropName, nameField, nameTextField);
        fieldBindingGroup.addBinding(nameBinding);

        areaBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ,
                field, fieldPropArea, areaField, textProperty);
        fieldBindingGroup.addBinding(areaBinding);

        validityStartBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE,
                field, fieldPropStartDate, validityStartField, formPropValidityStart);
        fieldBindingGroup.addBinding(validityStartBinding);

        validityEndBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE,
                field, fieldPropEndDate, validityEndField, formPropValidityEnd);
        fieldBindingGroup.addBinding(validityEndBinding);

        soilTypeBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE, 
                field, fieldPropSoilType, soilTypeComboBox, formPropSoilType);
        fieldBindingGroup.addBinding(soilTypeBinding);

        fieldBindingGroup.bind();
    }

    /**
     * Get the name field from this panel
     *
     * @return
     */
    public JTextField getNameField() {
        return nameField;
    }

    /**
     * Get the validity end field from this panel
     *
     * @return
     */
    public JXDatePicker getValidityEndField() {
        return validityEndField;
    }

    /**
     * Get the validity start field from this panel
     *
     * @return
     */
    public JXDatePicker getValidityStartField() {
        return validityStartField;
    }

    /**
     * Get the surface field from this panel
     *
     * @return
     */
    public JLabel getAreaField() {
        return areaField;
    }

    /**
     * Get the soil type combobox from this panel
     *
     * @return
     */
    public JComboBox getSoilTypeComboBox() {
        return soilTypeComboBox;
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        nameLabel = new javax.swing.JLabel();
        nameField = new JXTextField();
        validityLabel = new javax.swing.JLabel();
        validityStartField = new org.jdesktop.swingx.JXDatePicker();
        startEndateSeparator = new javax.swing.JLabel();
        validityEndField = new org.jdesktop.swingx.JXDatePicker();
        areaLabel = new javax.swing.JLabel();
        areaField = new HectaresLabel();
        soilTypeLabel = new javax.swing.JLabel();
        soilTypeComboBox = new javax.swing.JComboBox();

        setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1));

        nameLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        nameLabel.setText(org.openide.util.NbBundle.getMessage(FieldDataPanel.class, "FieldDataPanel.nameLabel.text")); // NOI18N
        nameLabel.setToolTipText(org.openide.util.NbBundle.getMessage(FieldDataPanel.class, "FieldDataPanel.nameLabel.toolTipText")); // NOI18N
        nameLabel.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);

        nameField.setText(org.openide.util.NbBundle.getMessage(FieldDataPanel.class, "FieldDataPanel.nameField.text")); // NOI18N

        validityLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        validityLabel.setText(org.openide.util.NbBundle.getMessage(FieldDataPanel.class, "FieldDataPanel.validityLabel.text")); // NOI18N

        startEndateSeparator.setText(org.openide.util.NbBundle.getMessage(FieldDataPanel.class, "FieldDataPanel.startEndateSeparator.text")); // NOI18N

        areaLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        areaLabel.setText(org.openide.util.NbBundle.getMessage(FieldDataPanel.class, "FieldDataPanel.areaLabel.text")); // NOI18N

        areaField.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        areaField.setText(org.openide.util.NbBundle.getMessage(FieldDataPanel.class, "FieldDataPanel.areaField.text")); // NOI18N
        areaField.setAlignmentX(0.2F);
        areaField.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 5, 1, 1));

        soilTypeLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        soilTypeLabel.setText(org.openide.util.NbBundle.getMessage(FieldDataPanel.class, "FieldDataPanel.soilTypeLabel.text")); // NOI18N

        soilTypeComboBox.setModel(soilTypeModel);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(soilTypeLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(nameLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(validityLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(areaLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(validityStartField, javax.swing.GroupLayout.PREFERRED_SIZE, 155, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(startEndateSeparator)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(validityEndField, javax.swing.GroupLayout.PREFERRED_SIZE, 160, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(nameField)
                    .addComponent(soilTypeComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(areaField, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(nameLabel)
                    .addComponent(nameField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(validityStartField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(validityLabel)
                    .addComponent(startEndateSeparator)
                    .addComponent(validityEndField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(areaLabel)
                    .addComponent(areaField))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(soilTypeLabel)
                    .addComponent(soilTypeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel areaField;
    private javax.swing.JLabel areaLabel;
    private javax.swing.JTextField nameField;
    private javax.swing.JLabel nameLabel;
    private javax.swing.JComboBox soilTypeComboBox;
    private javax.swing.JLabel soilTypeLabel;
    private javax.swing.JLabel startEndateSeparator;
    private org.jdesktop.swingx.JXDatePicker validityEndField;
    private javax.swing.JLabel validityLabel;
    private org.jdesktop.swingx.JXDatePicker validityStartField;
    // End of variables declaration//GEN-END:variables

    public class HectaresLabel extends JXLabel{

        @Override
        public void setText(String text) {
            super.setText(text+" "+Bundle.unit_abbreviation_hectares());
        }
        
    }

}
